/***************************************************************************
                          dialdialogimp.cpp  -  description
                             -------------------
    begin                : Wed Dec 26 2001
    copyright            : (C) 2001 by Robin Atwood
    email                : robin.atwood@attglobal.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialdialogimp.h"
//Added by qt3to4:
#include <QKeyEvent>

DialDialog::DialDialog( QWidget* parent, Qt::WFlags fl ) :
		Ui::DialDialog()															
{
	dialling = false;
	modem = 0;
	locked = 0;
  redialCount = 0;
	intvlLeft = 0;

	setupUi( this );
	dialDialButton->setFocus();
	dialDialButton->setIcon(KIcon("kall"));
	dialConfigButton->setIcon(KIcon("configure"));
	dialCancelButton->setIcon(KIcon("dialog-cancel"));
	dialHelpButton->setIcon(KIcon("help-contents"));

  KHelpMenu *helpMenu = new KHelpMenu(this, KGlobal::mainComponent().aboutData(), true);
  dialHelpButton->setMenu((QMenu*)helpMenu->menu());

	busyTimer = new QTimer( this );
	redialTimer = new QTimer( this );
	  				
  connect( busyTimer, SIGNAL( timeout() ), this, SLOT( slotBusyTimeout() ) );
  connect( redialTimer, SIGNAL( timeout() ), this, SLOT( slotRedialTimeout() ) );
	
	Q_UNUSED(parent);
	Q_UNUSED(fl);
}

DialDialog::~DialDialog()
{
	if ( locked )
		modem->unlockDevice();       						              // release modem lock if held
	if ( modem > 0 )
	{
		delete modem; modem = 0;
	}
}

void DialDialog::slotDialHangup()
{
	if ( modem == 0 )                                           // first entry
	{
		modem = new ModemComms( modemDevice.toLatin1() ); 				// get a modem
		locked = modem->lockDevice();
	}

	if ( !locked )
	{
		KMessageBox::sorry( this, i18n( "Unable to lock serial port.\nModem probably in use." ),
	        	i18n( "Warning" ) );
  	reject();
  }
	else
  {
  	if ( !dialling )
  	{                                       // Dial clicked
  		dialDialButton->setEnabled( false );
			setCursor( Qt::WaitCursor );
  		dialling = dialNumber( dialLocNum->text() );
  		if ( dialling )
  		{			
  			dialDialButton->setText( i18n( "&Hangup" ) );
  		  dialDialButton->setToolTip( i18n("Hangup the modem") );
  		  modem->setICanon( false );					// don't wait for newlines	
  		  busyTimer->start( 100 );     				// pop the busy checker every second
  			dialDialButton->setEnabled( true );
				dialDialButton->setFocus();
				setCursor( Qt::ArrowCursor );
  		}
  		else
  		{
  			reject();														// problem with serial port
  		}
  	}
  	else
  	{                                       // Hangup clicked
  		busyTimer->stop();
  		hangUp();
  	  accept();
  	}
	}
}

void DialDialog::slotDialCancel()
{
	if ( intvlLeft > 0 )
		redialTimer->stop();		
	if ( dialling )
	{
		busyTimer->stop();
		hangUp();
	}
	reject();
}

void DialDialog::slotDialConfig()
{
	ConfigDialogImp config( this, 0 );
	if ( config.exec() == QDialog::Accepted )
		kall->saveOptions();                  // save new options
}

void DialDialog::slotDialHelp() 
{
  KToolInvocation::invokeHelp();
}

void DialDialog::keyPressEvent( QKeyEvent *qkeyevent )
{
	int k = qkeyevent->key();
	if ( k == Qt::Key_Escape )
 	{
 		qkeyevent->accept();
 		slotDialCancel();
 	}
	else
		qkeyevent->ignore();
}

bool DialDialog::dialNumber( QString num )
{
 	if ( modem->initPort() == 0 )
 	{
	  if ( !modem->modemIO(modemInit1.toLatin1())  )
	  {
  		KMessageBox::error( this,
      			i18n( "No response from serial port.\nAre you sure there is a modem connected?" ),
      			i18n( "Severe error" ) );
     	modem->closePort();
    	return false;
	  }
   	modem->modemIO( modemInit2.toLatin1() );
   	modem->modemIO( QString( modemDial + num ).toLatin1() );
 	}
 	else
 	{
 		KMessageBox::error( this,
     			i18n("Unable to open serial port ") + modemDevice, i18n("Severe error" ) ) ;
   	return false;
  }
	return true;
}

void DialDialog::hangUp()
{
	setCursor(Qt::WaitCursor);
  modem->setICanon( true );						// wait for newlines	
 	modem->modemIO( QString("+++" + modemHangup).toLatin1() );
 	modem->closePort();
	setCursor(Qt::ArrowCursor);
}

void DialDialog::slotBusyTimeout()
{
  QString resp = QString( modem->getResp() ).trimmed();
	if ( resp == "BUSY" )
	{
		busyTimer->stop();
		dialling = false;
		dialDialButton->setEnabled( false );
	  hangUp();
	  ++redialCount;
    if ( redialCount > maxRetry )
	  {
	  	slotDialCancel();       // simulate user cancel
	  }
	  else
	  {
  	  retryText->setText( i18n("Redial attempt %1 of %2 in %3 seconds")
  	  				.arg(redialCount).arg(maxRetry).arg(intvlRetry) );
  	  redialTimer->start( 1000 );
			intvlLeft = intvlRetry;
	  }
	}
  else if ( resp == "NO DIALTONE" )
	{
		busyTimer->stop();
 		KMessageBox::error( this, i18n("No dial tone - check hand-set on-hook."),
				i18n("Severe error") );
		reject();
	}
}

void DialDialog::slotRedialTimeout()
{
	--intvlLeft;
	if ( intvlLeft == 0 )
	{
		redialTimer->stop();
		retryText->setText( "" );
		kapp->processEvents();
		slotDialHangup();						//	simulate user dial
	}
	else
	{
 	  retryText->setText( i18n("Redial attempt %1 of %2 in %3 seconds")
  	  				.arg(redialCount).arg(maxRetry).arg(intvlLeft) );
	}
}

void DialDialog::setDialLocNum( const QString &num )
{
	dialLocNum->setText( num );
}

