
/***************************************************************************
                          kall.cpp  -  description
                             -------------------
    begin                : Fri Dec 14 19:36:45 ICT 2001
    copyright            : (C) 2001 by Robin Atwood
    email                : robin.atwood@attglobal.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "configdialogimp.h"
#include "modemcomms.h"
#include "logviewdialogimp.h"
#include "statusdialogimp.h"
#include "kall.moc"
//Added by qt3to4:
#include <QPixmap>

using namespace KABC;
using namespace std;

KallApp::KallApp( QWidget* parent, KCmdLineArgs *args ) :
  KMainWindow( parent )
{
  kall = this;

  readOptions();

	number = args->getOption( "dial" );
	person = args->getOption( "person" );
	org    = args->getOption( "org" );
}

KallApp::~KallApp()
{
}

void KallApp::saveOptions()
{	
	// modem command entry
  modemGroup->writeEntry( "ModemInit1", modemInit1 );
	modemGroup->writeEntry( "ModemInit2", modemInit2 );
	modemGroup->writeEntry( "ModemDial", modemDial );
	modemGroup->writeEntry( "ModemHangup", modemHangup );
	modemGroup->writeEntry( "ModemDevice", modemDevice );
	modemGroup->config()->sync();
	
	// telephone code settings
	teleGroup->writeEntry( "iddCode", iddCode );
	teleGroup->writeEntry( "ldCode", ldCode );
	teleGroup->writeEntry( "localeCode", localeCode );
	teleGroup->writeEntry( "maxRetry", maxRetry );
	teleGroup->writeEntry( "intvlRetry", intvlRetry );
	teleGroup->config()->sync();
}


void KallApp::readOptions()
{
	bool ok;
	// modem command entry

	modemDeviceDflt = MODEM_DEVICE;
		
	modemDevice = modemGroup->readEntry( "ModemDevice", modemDeviceDflt );
	modemInit1  = modemGroup->readEntry( "ModemInit1", MODEM_INIT1 );
	modemInit2  = modemGroup->readEntry( "ModemInit2", MODEM_INIT2 );
	modemDial   = modemGroup->readEntry( "ModemDial", MODEM_DIAL );
	modemHangup = modemGroup->readEntry( "ModemHangup", MODEM_HANGUP );
	
	// telephone code settings

  iddCodeDflt = IDD_CODE;
  ldCodeDflt = LD_CODE;
  localeCodeDflt = LOCALE_CODE;

	if ( !config->hasGroup("Telephone") )
	{
		localeInfo();
		if ( localeCodeDflt == "1" ) ldCodeDflt = "1";  // weird US settings
	}

	iddCode = teleGroup->readEntry( "iddCode", iddCodeDflt );
	ldCode  = teleGroup->readEntry( "ldCode", ldCodeDflt );
	localeCode  = teleGroup->readEntry( "localeCode", localeCodeDflt );
	maxRetry = (teleGroup->readEntry( "maxRetry", "10" )).toInt(&ok);
	intvlRetry = (teleGroup->readEntry( "intvlRetry", "30" )).toInt(&ok);
}

int KallApp::showDialDialog()
{
 	QString num = number;
 	DialDialog *dialView = new DialDialog( this, 0 );   // create modal dialog
	findKABCentry();
 	localise( num );                    					           // add dialling codes
 	dialView->setDialLocNum( num );
 	int res = dialView->exec();
		
 	if ( res == QDialog::Accepted )
 	{
		StatusDialogImp statView( this, 0 );
    statView.showStats( person, number, org );		  		  	 // pops up timer dialog and writes log file
	}
	delete dialView; dialView = 0;  	
	return !res;
}

void KallApp::findKABCentry()
{
  AddressBook::Iterator entry;
  PhoneNumber::List phones;
	bool found = false;

	for ( entry = adbk->begin(); entry != adbk->end(); ++entry )
	{
    phones = (*entry).phoneNumbers();
    if ( phones.count() > 0 )
		{
      PhoneNumber::List::Iterator phone = phones.begin();
			for ( ; phone != phones.end(); ++phone )
      {
        if ( number == (*phone).number().replace( " ", "" ) ) 
				{
					found = true; break;
				}
			}
		}
		if ( found ) break;
	}

	if ( found )
	{
		QString fn, mn, ln;
   	fn = (*entry).givenName().trimmed();
   	mn = (*entry).additionalName().trimmed();
   	ln = (*entry).familyName().trimmed();
		person = ln;
		if ( fn != "" )
			person += ", " + fn + " " + mn;
    if ( person == "" )
			person = (*entry).name().trimmed();
		org = (*entry).organization();
	}
}

/*  output produced by locale
int_select="00"\n
int_prefix="44"\n
0123456789012345
*/

void KallApp::localeInfo()
{
  FILE *fp;  QString tmp; int l;
  char line[130];																	/* line of data from unix command*/
  const int OFF = 12;
  fp = popen( "locale -ck LC_TELEPHONE", "r" );		/* Issue the command.		*/

  while ( fgets( line, sizeof( line ), fp ) )
  {
		tmp = line;    	
    if ( tmp.startsWith( "int_select" ) )
    {
   		l = tmp.length();		
   		iddCodeDflt = tmp.mid( OFF, l-OFF-2 );
    }
    else if ( tmp.startsWith( "int_prefix" ) )
    {
   		l = tmp.length();		
   		localeCodeDflt = tmp.mid( OFF, l-OFF-2 );
    }
  }
  pclose(fp);
}
